﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;
using AForge.Vision.GlyphRecognition;
using Microsoft.Research.Kinect.Nui;
using AForge;
using AForge.Math.Geometry;

namespace DepthTracker
{
    public class GlyphDetector
    {
        private GlyphRecognizer m_Recognizer;

        public GlyphDetector(int size)
        {
            m_Recognizer = new GlyphRecognizer(size);
            m_Recognizer.GlyphDatabase = new GlyphDatabase(size);
        }

        public void AddGlyph(string name, string data)
        {
            Glyph glyph = new Glyph(name, GlyphDataFromString(data, m_Recognizer.GlyphSize));
            m_Recognizer.GlyphDatabase.Add(glyph);
        }

        public List<HotZoneSpec> Process(ImageFrame imageFrame)
        {
            // Prepare a bitmap
            Bitmap bitmap = PImageToBitmap(imageFrame.Image);
            bitmap.RotateFlip(RotateFlipType.Rotate180FlipY);

            // Find glyphs
            List<ExtractedGlyphData> glyphs = m_Recognizer.FindGlyphs(bitmap);
            List<HotZoneSpec> hotzones = new List<HotZoneSpec>();

            // Convert found glyphs to a list of hotzones
            foreach (ExtractedGlyphData egd in glyphs)
            {
                IntPoint minXY, maxXY;
                PointsCloud.GetBoundingRectangle(egd.Quadrilateral, out minXY, out maxXY);
                HotZoneSpec hz = new HotZoneSpec(egd.RecognizedGlyph.Name, minXY.X, minXY.Y, maxXY.X - minXY.X, maxXY.Y - minXY.Y, 0, 0);
                hz.Confidence = Math.Round(egd.Confidence * 100.0, 0);
                hotzones.Add(hz);
            }

            return hotzones;
        }

        #region Helpers

        private Bitmap PImageToBitmap(PlanarImage PImage)
        {
            Bitmap bmap = new Bitmap(PImage.Width, PImage.Height, PixelFormat.Format32bppRgb);
            BitmapData bmapdata = bmap.LockBits(new System.Drawing.Rectangle(0, 0, PImage.Width, PImage.Height),
                ImageLockMode.WriteOnly, bmap.PixelFormat);

            IntPtr ptr = bmapdata.Scan0;
            System.Runtime.InteropServices.Marshal.Copy(PImage.Bits, 0, ptr, PImage.Width * PImage.BytesPerPixel * PImage.Height);
            bmap.UnlockBits(bmapdata);

            return bmap;
        }

        private byte[,] GlyphDataFromString(string glyphStrData, int glyphSize)
        {
            byte[,] glyphData = new byte[glyphSize, glyphSize];

            for (int i = 0, k = 0; i < glyphSize; i++)
            {
                for (int j = 0; j < glyphSize; j++, k++)
                {
                    glyphData[i, j] = (byte)((glyphStrData[k] == '0') ? 0 : 1);
                }
            }

            return glyphData;
        }

        #endregion
    }
}
