﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;

namespace DepthTracker
{
    public class HotZone
    {
        private string m_Name;
        private Rect m_Rectangle;
        private double m_MinZ;
        private double m_MaxZ;
        private double m_Area;
        private double m_HitCount;
        private double m_MissingCount;
        TextBlock m_TextBlock;

        public HotZone(string name, double x, double y, double width, double height, double minZ, double maxZ)
        {
            m_Name = name;
            m_Rectangle = new Rect(x, y, width, height);
            m_MinZ = minZ;
            m_MaxZ = maxZ;
            m_Area = (width + 1) * (height + 1);
        }

        public void Draw(Canvas canvas, double scale)
        {
            Draw(canvas, scale, Brushes.White);
        }

        public void Draw(Canvas canvas, double scale, Brush stroke)
        {
            System.Windows.Shapes.Rectangle rect = new System.Windows.Shapes.Rectangle();
            rect.Stroke = stroke;

            rect.Width = m_Rectangle.Width * scale;
            rect.Height = m_Rectangle.Height * scale;
            Canvas.SetLeft(rect, m_Rectangle.Left * scale);
            Canvas.SetTop(rect, m_Rectangle.Top * scale);
            canvas.Children.Add(rect);

            m_TextBlock = new TextBlock();
            m_TextBlock.Text = m_Name;
            m_TextBlock.Foreground = stroke;
            m_TextBlock.Width = rect.Width;
            m_TextBlock.Height = 44;
            m_TextBlock.TextWrapping = TextWrapping.Wrap;
            m_TextBlock.TextAlignment = TextAlignment.Center;
            Canvas.SetLeft(m_TextBlock, m_Rectangle.Left * scale);
            Canvas.SetTop(m_TextBlock, (m_Rectangle.Top * scale) + ((rect.Height - m_TextBlock.Height) / 2));
            canvas.Children.Add(m_TextBlock);
        }

        public bool Contains(double x, double y, double depth)
        {
            if (m_Rectangle.Contains(x, y))
            {
                if (depth == 0)
                {
                    m_MissingCount++;
                    return false;
                }
                else if (depth > m_MinZ && depth < m_MaxZ)
                {
                    m_HitCount++;
                    return true;
                }
            }

            return false;
        }

        public string Name
        {
            get { return m_Name; }
        }

        public double HitPercent
        {
            get { return Math.Truncate((m_HitCount / m_Area) * 100); }
            set { m_HitCount = value; m_MissingCount = value; }
        }

        public double MissingPercent
        {
            get { return Math.Truncate((m_MissingCount / m_Area) * 100); }
            set { m_HitCount = value; m_MissingCount = value; }
        }

        public string Text
        {
            get { return m_TextBlock.Text; }
            set { m_TextBlock.Text = value; }
        }
    }
}
