﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using Microsoft.Research.Kinect.Nui;
using Coding4Fun.Kinect.Wpf;

namespace DepthTracker
{
    public class ObstacleDetector
    {
        List<HotZoneSpec> m_HotZones;

        public struct ScanResult
        {
            public double X;
            public double Y;
            public double Distance { get; set; }
            public double ImageFrameWidth;
            public double ImageFrameHeight;
            public bool Hit;
        }

        public delegate void ScanHandler(ScanResult result);
        public event ScanHandler Scan;

        public ObstacleDetector()
        {
            m_HotZones = new List<HotZoneSpec>();
        }

        public HotZoneSpec AddHotZone(string name, double x, double y, double width, double height, double minZ, double maxZ)
        {
            HotZoneSpec hz = new HotZoneSpec(name, x, y, width, height, minZ, maxZ);
            m_HotZones.Add(hz);
            return hz;
        }

        public void RemoveHotZone(string name)
        {
            foreach (HotZoneSpec hz in m_HotZones)
            {
                if (hz.Name == name)
                    m_HotZones.Remove(hz);
            }
        }

        public List<HotZoneSpec> Process(ImageFrame imageFrame)
        {
            List<HotZoneSpec> hits = new List<HotZoneSpec>();

            // Reset hot zone counters
            foreach (HotZoneSpec hz in m_HotZones)
                hz.ResetCounters();

            int height = imageFrame.Image.Height;
            int width = imageFrame.Image.Width;

            // Depth data for each pixel
            Byte[] depthData = imageFrame.Image.Bits;

            // Scan all pixels and look for hits in the hot zones
            var depthIndex = 0;
            for (var y = 0; y < height; y++)
            {
                var heightOffset = y * width;
                for (var x = 0; x < width; x++)
                {
                    var index = (x + heightOffset) * 4;
                    var distance = GetDistance(depthData[depthIndex], depthData[depthIndex + 1]);

                    bool hit = false;
                    foreach (HotZoneSpec hz in m_HotZones)
                    {
                        if (hz.Contains(x, y, distance))
                        {
                            hit = true;
                            if (!hits.Contains(hz))
                                hits.Add(hz);
                        }
                    }

                    if (Scan != null)
                        Scan(new ScanResult() { X = x, Y = y, Distance = distance, Hit = hit, ImageFrameWidth = imageFrame.Image.Width, ImageFrameHeight = imageFrame.Image.Height });

                    // Move two bytes at a time
                    depthIndex += 2;
                }
            }

            return hits;
        }

        public List<HotZoneSpec> HotZones
        {
            get { return m_HotZones; }
        }

        private int GetDistance(byte firstFrame, byte secondFrame)
        {
            int distance = (int)firstFrame | secondFrame << 8;
            return distance;
        }
    }
}
