﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;
using System.IO.Ports;
using KBotController;

namespace TestHarness
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        Controller m_KBot;

        private DispatcherTimer m_timerTelem;
        private int m_TelReqTick;
        private int m_TelReqInterval = 5;
        private bool m_AllowClick = true;

        public MainWindow()
        {
            InitializeComponent();

            try
            {
                Rect bounds = Properties.Settings.Default.WinPosition;
                this.Top = bounds.Top;
                this.Left = bounds.Left;
                this.Height = bounds.Height;
                this.Width = bounds.Width;
            }
            catch
            {
            }
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            // Load com ports
            List<string> ports = SerialPort.GetPortNames().ToList<string>();
            foreach (string port in ports)
            {
                cbComPort.Items.Add(port);
            }
            cbComPort.SelectedItem = Properties.Settings.Default.ComPort;

            // Setup telemetry request timer
            m_timerTelem = new DispatcherTimer();
            m_timerTelem.Interval = TimeSpan.FromMilliseconds(100);
            m_timerTelem.Tick += new EventHandler(AutoTelemetryHandler);
            m_TelReqTick = m_TelReqInterval;
            m_timerTelem.Start();
        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            Properties.Settings.Default.ComPort = (string)cbComPort.SelectedItem;
            Properties.Settings.Default.WinPosition = this.RestoreBounds;
            Properties.Settings.Default.Save();
        }

        private void btnClose_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }

        private void cbComPort_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (m_KBot != null)
                m_KBot.Dispose();

            m_KBot = new Controller((string)cbComPort.SelectedItem);
        }

        private void btnForward_Click(object sender, RoutedEventArgs e)
        {
            if (string.IsNullOrWhiteSpace(txtFwdDist.Text))
                m_KBot.MoveForward();
            else
                m_KBot.MoveForward(int.Parse(txtFwdDist.Text));
        }

        private void btnStop_Click(object sender, RoutedEventArgs e)
        {
            if ((bool)cbEmergency.IsChecked)
                m_KBot.EmergencyHalt();
            else
                m_KBot.Halt();
        }

        private void btnBack_Click(object sender, RoutedEventArgs e)
        {
            m_KBot.MoveReverse();
        }

        private void btnLeft_Click(object sender, RoutedEventArgs e)
        {
            if (string.IsNullOrWhiteSpace(txtLeftBias.Text))
                m_KBot.TurnLeft(0);
            else
                m_KBot.TurnLeft(int.Parse(txtLeftBias.Text));
        }

        private void btnRight_Click(object sender, RoutedEventArgs e)
        {
            if (string.IsNullOrWhiteSpace(txtRightBias.Text))
                m_KBot.TurnRight(0);
            else
                m_KBot.TurnRight(int.Parse(txtRightBias.Text));
        }

        private void btnSlow_Click(object sender, RoutedEventArgs e)
        {
            m_KBot.SetSpeed(1);
        }

        private void btnMedium_Click(object sender, RoutedEventArgs e)
        {
            m_KBot.SetSpeed(2);
        }

        private void btnFast_Click(object sender, RoutedEventArgs e)
        {
            m_KBot.SetSpeed(3);
        }

        private void btnPower1_Checked(object sender, RoutedEventArgs e)
        {
            if (m_AllowClick)
                m_KBot.SetPower(true);
        }

        private void btnPower1_Unchecked(object sender, RoutedEventArgs e)
        {
            if (m_AllowClick)
                m_KBot.SetPower(false);
        }

        private void btnKinectPos_Click(object sender, RoutedEventArgs e)
        {
            if (string.IsNullOrWhiteSpace(txtKinectPos.Text))
                m_KBot.SetKinectPosition(90);
            else
                m_KBot.SetKinectPosition(int.Parse(txtKinectPos.Text));
        }

        private void btnGetTel_Click(object sender, RoutedEventArgs e)
        {
            GetTelemetry();
        }

        private void GetTelemetry()
        {
            m_KBot.UpdateSensorData();
            txtIRLeft.Text = m_KBot.IRLeft.ToString();
            txtIRRight.Text = m_KBot.IRRight.ToString();
            txtPingLeft.Text = m_KBot.PingLeft.ToString();
            txtPingCenter.Text = m_KBot.PingCenter.ToString();
            txtPingRight.Text = m_KBot.PingRight.ToString();

            m_KBot.UpdateStatus();
            txtSensorKinectPos.Text = m_KBot.KinectPosition.ToString();
            txtDrivePower.Text = m_KBot.DrivePower ? "On" : "Off";
            txtSpeed.Text = m_KBot.CurrentSpeed.ToString();
            txtDirection.Text = m_KBot.CurrentDirection.ToString();
            txtWheelPosLeft.Text = m_KBot.WheelPositionLeft.ToString();
            txtWheelPosRight.Text = m_KBot.WheelPositionRight.ToString();
            txtHasArrived.Text = m_KBot.HasArrived ? "Yes" : "No";

            m_AllowClick = false;
            btnPower1.IsChecked = m_KBot.DrivePower;
            m_AllowClick = true;
        }

        private void AutoTelemetryHandler(Object sender, EventArgs args)
        {
            if ((bool)cbAutoReqTel.IsChecked)
            {
                m_TelReqTick -= 1;
                if (m_TelReqTick == 0)
                {
                    GetTelemetry();
                    m_TelReqTick = m_TelReqInterval;
                }
            }
        }
    }
}
