﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.IO.Ports;
using System.Windows.Threading;
using System.Threading;
using Microsoft.Research.Kinect.Nui;
using Coding4Fun.Kinect.Wpf;
using System.Windows.Controls.Primitives;

namespace KBotTracker
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private Cortex m_KBot;
        private DispatcherTimer m_timerTrack;

        Cortex.TrackingResult m_Result;

        public MainWindow()
        {
            InitializeComponent();

            try
            {
                Rect bounds = Properties.Settings.Default.WinPosition;
                this.Top = bounds.Top;
                this.Left = bounds.Left;
                this.Height = bounds.Height;
                this.Width = bounds.Width;
            }
            catch
            {
            }
        }

        #region Event Handlers

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            slTilt.Value = Properties.Settings.Default.CameraAngle;

            // Load com ports
            List<string> ports = SerialPort.GetPortNames().ToList<string>();
            foreach (string port in ports)
            {
                cbComPort.Items.Add(port);
            }
            cbComPort.SelectedItem = Properties.Settings.Default.ComPort;

            // Setup tracking timer
            m_timerTrack = new DispatcherTimer();
            m_timerTrack.Interval = TimeSpan.FromMilliseconds(500);
            m_timerTrack.Tick += new EventHandler(TrackTimerHandler);
            m_timerTrack.Start();
        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            Properties.Settings.Default.ComPort = (string)cbComPort.SelectedItem;
            Properties.Settings.Default.WinPosition = this.RestoreBounds;
            Properties.Settings.Default.Save();

            if(m_KBot != null)
                m_KBot.Dispose();
        }

        private void btnClose_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }

        private void cbComPort_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (m_KBot != null)
                m_KBot.Dispose();

            m_KBot = new Cortex((string)cbComPort.SelectedItem, (int)slTilt.Value);
            m_KBot.VisualTrack += new Cortex.TrackingHandler(VisualTrackHandler);
            m_KBot.AudioTrack += new Cortex.TrackingHandler(AudioTrackHandler);

            cbAutoReqTel.IsChecked = true;
        }

        private void cbSensorScan_Click(object sender, RoutedEventArgs e)
        {
            m_KBot.SetSensorScan((bool)cbSensorScan.IsChecked);
        }

        private void btnGetTel_Click(object sender, RoutedEventArgs e)
        {
            GetTelemetry();
        }

        private void rbTrackVisual_Checked(object sender, RoutedEventArgs e)
        {
            if (m_KBot != null)
                m_KBot.CurrentTrackingMode = Cortex.TrackingMode.Visual;
        }

        private void rbTrackAudio_Checked(object sender, RoutedEventArgs e)
        {
            if (m_KBot != null)
                m_KBot.CurrentTrackingMode = Cortex.TrackingMode.Audio;
        }

        private void rbTrackNone_Checked(object sender, RoutedEventArgs e)
        {
            if (m_KBot != null)
                m_KBot.CurrentTrackingMode = Cortex.TrackingMode.None;
        }

        private void cbActiveTracking_Click(object sender, RoutedEventArgs e)
        {
            m_KBot.ActiveTracking = (bool)cbActiveTracking.IsChecked;
        }

        private void btnTilt_Click(object sender, RoutedEventArgs e) 
        {
            if (m_KBot != null)
            {
                m_KBot.SetCameraAngle((int)slTilt.Value);
                Properties.Settings.Default.CameraAngle = slTilt.Value;
                Properties.Settings.Default.Save();
            }
        }

        private void cbUsingZoom_Click(object sender, RoutedEventArgs e)
        {
            if (m_KBot != null)
                m_KBot.UsingZoom = (bool)cbUsingZoom.IsChecked;
        }

        private void TrackTimerHandler(object sender, EventArgs e)
        {
            if ((bool)cbAutoReqTel.IsChecked)
            {
                GetTelemetry();
            }
        }

        private void VisualTrackHandler(Cortex.TrackingResult result)
        {
            m_Result = result;
            Dispatcher.BeginInvoke(new ThreadStart(TrackHandlerDisplay));
        }

        private void AudioTrackHandler(Cortex.TrackingResult result)
        {
            m_Result = result;
            Dispatcher.BeginInvoke(new ThreadStart(TrackHandlerDisplay));
        }

        private void TrackHandlerDisplay()
        {
            // Update Telemetry tab
            if (m_Result.ResultType == Cortex.TrackingMode.Visual)
            {
                txtTrackStatus.Text = m_Result.Status.ToString() + ", " + m_Result.Quality.ToString() + " (ID=" + m_Result.TrackingID.ToString() + ") ";
                txtTrackBearing.Text = m_Result.Bearing.ToString() + "/" + m_Result.Distance.ToString();
            }
            else if (m_Result.ResultType == Cortex.TrackingMode.Audio)
            {
                txtAudioTrack.Text = m_KBot.LastAudioTrack.Bearing.ToString();
            }

            if (m_Result.Status != Cortex.TrackStatus.Tracking)
                txtTrackStatus.Background = Brushes.Yellow;
            else
                txtTrackStatus.Background = Brushes.White;

            // Update Tracking tab if selected
            if (tabControl.SelectedIndex == 1)
            {
                SetThumbData(slBearing, Brushes.Black, txtTrackStatus.Background, m_Result.Bearing);
                SetThumbData(slServoPos, Brushes.Black, txtTrackStatus.Background, m_KBot.CurrentServoPos);
            }
        }

        #endregion

        #region Helpers

        private void SetThumbData(Slider slider, Brush foreground, Brush background, int value)
        {
            slider.Value = value;

            var track = slider.Template.FindName("PART_Track", slider) as Track;
            var thumb = track.Thumb;
            
            var el = thumb.Template.FindName("Ellipse", thumb) as Ellipse;
            el.Fill = background;

            var txt = thumb.Template.FindName("Text", thumb) as TextBlock;
            txt.Text = value.ToString();
            txt.Foreground = foreground;
            txt.Background = background;
        }

        private void GetTelemetry()
        {
            txtIRLeft.Text = m_KBot.IRLeft.ToString();
            txtIRRight.Text = m_KBot.IRRight.ToString();
            txtPingLeft.Text = m_KBot.PingLeft.ToString();
            txtPingCenter.Text = m_KBot.PingCenter.ToString();
            txtPingRight.Text = m_KBot.PingRight.ToString();

            txtDrivePower.Text = m_KBot.DrivePower ? "On" : "Off";
            txtSpeed.Text = m_KBot.CurrentSpeed.ToString();
            txtDirection.Text = m_KBot.CurrentDirection.ToString();
            txtWheelPosLeft.Text = m_KBot.WheelPositionLeft.ToString();
            txtWheelPosRight.Text = m_KBot.WheelPositionRight.ToString();
            txtHasArrived.Text = m_KBot.HasArrived ? "Yes" : "No";
        }

        #endregion
    }
}
