﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;

namespace KSeeM1
{
    public struct HotZoneSpec
    {
        private string m_Name;
        private Rect m_Rectangle;
        private double m_MinZ;
        private double m_MaxZ;
        private double m_Area;
        private double m_HitCount;
        private double m_MissingCount;
        private double m_Confidence;
        private double m_CameraBearing;
        private double m_RobotBearing;
        private double m_Distance;

        public HotZoneSpec(string name, double x, double y, double width, double height, double minZ, double maxZ)
        {
            m_Name = name;
            m_Rectangle = new Rect(x, y, width, height);
            m_MinZ = minZ;
            m_MaxZ = maxZ;
            m_Area = (width + 1) * (height + 1);
            m_HitCount = 0;
            m_MissingCount = 0;
            m_Confidence = 1;
            m_CameraBearing = 0;
            m_RobotBearing = 0;
            m_Distance = 0;
        }

        public bool Contains(double x, double y, double depth)
        {
            if (m_Rectangle.Contains(x, y))
            {
                if (depth == 0)
                {
                    m_MissingCount++;
                    return false;
                }
                else if (depth > m_MinZ && depth < m_MaxZ)
                {
                    m_HitCount++;
                    return true;
                }
            }

            return false;
        }

        public void ResetCounters()
        {
            m_HitCount = 0;
            m_MissingCount = 0;
        }

        public string Name
        {
            get { return m_Name; }
        }

        public double HitPercent
        {
            get { return (m_HitCount / m_Area) * 100; }
        }

        public double MissingPercent
        {
            get { return (m_MissingCount / m_Area) * 100; }
        }

        public double Confidence
        {
            get { return m_Confidence; }
            set { m_Confidence = value; }
        }

        public Rect Rectangle
        {
            get { return m_Rectangle; }
        }

        public double MinZ
        {
            get { return m_MinZ; }
        }

        public double MaxZ
        {
            get { return m_MaxZ; }
        }

        public double CameraBearing
        {
            get { return m_CameraBearing; }
            set { m_CameraBearing = value; }
        }

        public double RobotBearing
        {
            get { return m_RobotBearing; }
            set { m_RobotBearing = value; }
        }

        public double Distance
        {
            get { return m_Distance; }
            set { m_Distance = value; }
        }

        public Point Center
        {
            get { return new Point(m_Rectangle.Left + m_Rectangle.Width / 2, m_Rectangle.Top + m_Rectangle.Height / 2); }
        }
    }
}
