﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Research.Kinect.Nui;
using Coding4Fun.Kinect.Wpf; 

namespace SkeletalTracking
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        // Kinect Runtime
        Runtime nui = Runtime.Kinects[0];

        public MainWindow()
        {
            InitializeComponent();

            try
            {
                Rect bounds = Properties.Settings.Default.WinPosition;
                this.Top = bounds.Top;
                this.Left = bounds.Left;
                this.Height = bounds.Height;
                this.Width = bounds.Width;
            }
            catch
            {
            }
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {            

            //Initialize to do skeletal tracking
            nui.Initialize(RuntimeOptions.UseSkeletalTracking);

            slTilt.Value = Properties.Settings.Default.CameraAngle;
            nui.NuiCamera.ElevationAngle = (int)slTilt.Value;

            #region TransformSmooth
            //Must set to true and set after call to Initialize
            nui.SkeletonEngine.TransformSmooth = true;

            //Use to transform and reduce jitter
            var parameters = new TransformSmoothParameters
            {
                Smoothing = 0.75f,
                Correction = 0.0f,
                Prediction = 0.0f,
                JitterRadius = 0.05f,
                MaxDeviationRadius = 0.04f
            };

            nui.SkeletonEngine.SmoothParameters = parameters; 

            #endregion

            //add event to receive skeleton data
            nui.SkeletonFrameReady += new EventHandler<SkeletonFrameReadyEventArgs>(nui_SkeletonFrameReady);

        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            // Cleanup
            nui.Uninitialize();

            Properties.Settings.Default.WinPosition = this.RestoreBounds;
            Properties.Settings.Default.Save();
        }

        private void btnTilt_Click(object sender, RoutedEventArgs e)
        {
            nui.NuiCamera.ElevationAngle = (int)slTilt.Value;
            Properties.Settings.Default.CameraAngle = slTilt.Value;
            Properties.Settings.Default.Save();
        }

        void nui_SkeletonFrameReady(object sender, SkeletonFrameReadyEventArgs e)
        {
            
            SkeletonFrame allSkeletons = e.SkeletonFrame;

            txtFrameFloor.Text = allSkeletons.FloorClipPlane.W.ToString();
            txtFrameQuality.Text = allSkeletons.Quality.ToString();

            //get the first tracked skeleton
            SkeletonData skeleton = (from s in allSkeletons.Skeletons
                                     where s.TrackingState == SkeletonTrackingState.Tracked
                                     select s).FirstOrDefault();

            if (skeleton != null)
            {
                MainCanvas.Background = Brushes.White;
                txtTrackingID.Text = skeleton.TrackingID.ToString();
                txtQuality.Text = skeleton.Quality.ToString();
                txtTrackingState.Text = skeleton.TrackingState.ToString();

                //set position
                SetEllipsePosition(grdHead, txtHead, skeleton.Joints[JointID.Head]);
                SetEllipsePosition(grdLeft, txtLeft, skeleton.Joints[JointID.HandLeft]);
                SetEllipsePosition(grdRight, txtRight, skeleton.Joints[JointID.HandRight]);
                SetEllipsePosition(grdCenter, txtCenter, skeleton.Joints[JointID.ShoulderCenter]);
                SetEllipsePosition(grdSpine, txtSpine, skeleton.Joints[JointID.Spine]);
            }
            else
            {
                MainCanvas.Background = Brushes.Gray;
                txtTrackingID.Text = "";
                txtQuality.Text = "";
                txtTrackingState.Text = "";
            }
        }

        private void SetEllipsePosition(FrameworkElement element, TextBlock tb, Joint joint)
        {
            var scaledJoint = joint.ScaleTo(640, 480, .5f, .5f);
//            var scaledJoint2 = joint.ScaleTo(60, 480, .5f, .5f);

//            tb.Text = joint.TrackingState.ToString();
//            tb.Text = Meters2Inches(joint.Position.X).ToString();
//            tb.Text = Math.Round(30 - scaledJoint2.Position.X,0).ToString();
            tb.Text = Meters2Inches(joint.Position.Z, (bool)cbUsingZoom.IsChecked).ToString() + "\"";

            if (joint.TrackingState == JointTrackingState.Tracked)
                tb.Background = Brushes.White;
            else
                tb.Background = Brushes.Silver;

            Canvas.SetLeft(element, scaledJoint.Position.X);
            Canvas.SetTop(element, scaledJoint.Position.Y);
        }

        private double Meters2Inches(float meters, bool usingZoom)
        {
            double inches = Math.Round(((double)meters * 39.37), 1);
            if (usingZoom)
                return inches - (inches * .25);
            else
                return inches;
        }
    }
}
